@@ -1,20 +1,20 @@
 from django import forms
-from django.forms import models as modelforms
-from django.db.models import Q, get_model
 from django.contrib.gis.forms import fields
-from django.utils.translation import ugettext_lazy as _
-from django.contrib.gis.geoip import HAS_GEOIP
-from django.conf import settings
+from django.db.models import Q
+from django.forms import models as modelforms
+from django.utils.encoding import force_text
+from django.utils.translation import gettext_lazy as _
+from oscar.core.loading import get_model
 
 OpeningPeriod = get_model('stores', 'OpeningPeriod')
-assert OpeningPeriod
 
 
 class StoreAddressForm(forms.ModelForm):
 
     class Meta:
         model = get_model('stores', 'StoreAddress')
-        exclude = ('title', 'first_name', 'last_name', 'search_text')
+        fields = [
+            'line1', 'line2', 'line3', 'line4', 'state', 'postcode', 'country']
 
 
 class StoreForm(forms.ModelForm):
@@ -22,25 +22,21 @@ class StoreForm(forms.ModelForm):
 
     class Meta:
         model = get_model('stores', 'Store')
-        exclude = ('slug', 'opening_periods', )
+        fields = [
+            'name', 'manager_name', 'phone', 'email', 'reference', 'image',
+            'description', 'location', 'group', 'is_pickup_store', 'is_active',
+        ]
         widgets = {
             'description': forms.Textarea(attrs={'cols': 40, 'rows': 10}),
         }
 
     def __init__(self, *args, **kwargs):
-        current_ip = kwargs.pop('current_ip', None)
-        super(StoreForm, self).__init__(*args, **kwargs)
-
+        super().__init__(*args, **kwargs)
         # Make sure that we store the initial data as GeoJSON so that
         # it is easier for us to use it in Javascript.
         instance = kwargs.get('instance', None)
         if instance:
             self.initial['location'] = instance.location.geojson
-        elif HAS_GEOIP and getattr(settings, 'GEOIP_ENABLED', True):
-            from django.contrib.gis.geoip import GeoIP
-            point = GeoIP().geos(current_ip)
-            if point:
-                self.initial['location'] = point.geojson
 
     def clean_reference(self):
         ref = self.cleaned_data['reference']
@@ -51,17 +47,9 @@ class StoreForm(forms.ModelForm):
 
 class OpeningPeriodForm(forms.ModelForm):
 
-    def __init__(self, *args, **kwargs):
-        super(OpeningPeriodForm, self).__init__(*args, **kwargs)
-        time_input = ['%H:%M', '%H', '%I:%M%p', '%I%p', '%I:%M %p', '%I %p']
-        self.fields['start'].input_formats = time_input
-        self.fields['start'].required = False
-        self.fields['end'].input_formats = time_input
-        self.fields['end'].required = False
-
     class Meta:
         model = OpeningPeriod
-        exclude = ('store', 'weekday', )
+        fields = ['start', 'end']
         widgets = {
             'name': forms.TextInput(
                 attrs={'placeholder': _("e.g. Christmas")}
@@ -76,6 +64,22 @@ class OpeningPeriodForm(forms.ModelForm):
             ),
         }
 
+    def __init__(self, *args, **kwargs):
+        self.weekday = kwargs.pop('weekday')
+        self.store = kwargs.pop('store')
+
+        super().__init__(*args, **kwargs)
+        time_input = ['%H:%M', '%H', '%I:%M%p', '%I%p', '%I:%M %p', '%I %p']
+        self.fields['start'].input_formats = time_input
+        self.fields['start'].required = False
+        self.fields['end'].input_formats = time_input
+        self.fields['end'].required = False
+
+    def save(self, commit=True):
+        self.instance.store = self.store
+        self.instance.weekday = self.weekday
+        return super().save(commit=commit)
+
 
 class DashboardStoreSearchForm(forms.Form):
     name = forms.CharField(label=_('Store name'), required=False)
@@ -108,15 +112,21 @@ class IsOpenForm(forms.Form):
         self.is_valid()
         return self.cleaned_data['open']
 
+    def __bool__(self):
+        return self.__nonzero__()
+
 
 class OpeningPeriodFormset(modelforms.BaseInlineFormSet):
     extra = 10
     can_order = False
     can_delete = True
-    max_num = 30 # Reasonably safe number of maximum period intervals per day
+    min_num = 0
+    max_num = 30  # Reasonably safe number of maximum period intervals per day
     absolute_max = 30
-    fk = [f for f in OpeningPeriod._meta.fields if f.name == 'store'][0]
+    fk = [f for f in OpeningPeriod._meta.get_fields() if f.name == 'store'][0]
+    form = OpeningPeriodForm
     model = OpeningPeriod
+    validate_min = True
     validate_max = True
 
     def __init__(self, weekday, data, instance):
@@ -133,21 +143,16 @@ class OpeningPeriodFormset(modelforms.BaseInlineFormSet):
 
         self.open = self.openform['open']
 
-        super(OpeningPeriodFormset, self).__init__(data=data, instance=instance,
-                                                   prefix=prefix,
-                                                   queryset=queryset)
+        super().__init__(data=data, instance=instance, prefix=prefix, queryset=queryset)
 
     def get_weekday_display(self):
-        return unicode(OpeningPeriod.WEEK_DAYS[self.weekday])
+        return force_text(OpeningPeriod.WEEK_DAYS[self.weekday])
 
-    def form(self, *args, **kwargs):
-         form = OpeningPeriodForm(*args, **kwargs)
-         form.instance.weekday = self.weekday
-         form.instance.store = self.instance
-         return form
-
-    def is_valid(self):
-        return super(OpeningPeriodFormset, self).is_valid()
+    def get_form_kwargs(self, index):
+        return {
+            'store': self.instance,
+            'weekday': self.weekday,
+        }
 
     def save(self, *args, **kwargs):
         if not self.openform:
@@ -155,7 +160,7 @@ class OpeningPeriodFormset(modelforms.BaseInlineFormSet):
                 if form.instance.pk:
                     form.instance.delete()
         else:
-            return super(OpeningPeriodFormset, self).save(*args, **kwargs)
+            return super().save(*args, **kwargs)
 
 
 class OpeningHoursFormset(object):
@@ -163,8 +168,7 @@ class OpeningHoursFormset(object):
         self.data = data or None
         self.instance = instance
         self.forms = [self.construct_sub_formset(weekday) for weekday in
-            OpeningPeriod.WEEK_DAYS
-        ]
+                      OpeningPeriod.WEEK_DAYS]
 
     def __iter__(self):
         return iter(self.forms)
